<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class Bar extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_BAR = "596507668cde464323c18d7a";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getXAxis(){
        $widget = $this->widget;
        $xAxis = [
            "type" => "value",
            "axisLabel" => [
                "show" => true,
                "rotate" => !empty($widget['chartXAxisTicksRotation']) ? $widget['chartXAxisTicksRotation'] : 0,
            ],
        ];
        return $xAxis;
    }

    protected function getYAxis(){
        $widget = $this->widget;
        $data = $this->widgetData;
        $categoria = [];
        foreach ($data as $row) {
            $categoria[] = $row[$widget['chartXAxisField']];
        }
        $yAxis = [
            "type" => "category",
            "data" => $categoria,
        ];

        return $yAxis;
    }

    protected function getSeries(){
        $widget = $this->widget;
        $series = [];
        foreach ($this->widgetData as $row) {
            foreach ($widget['serieFields'] as $index => $fact) {
                if(empty($series[$index])){
                    $series[$index] = array(
                        "name" => $widget['serieLabels'][$index] ,
                        "type" => "bar",
                        "barWidth" => $this->getBarWidth(),
                        "data" => array()
                    );
                }
                $series[$index]["data"][] = !empty($row[$fact]) ? $row[$fact] : 0;
            }
        }
        return $series;
    }

    protected function getTooltip(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["chartShowTooltip"]) ? $widget["chartShowTooltip"] : false
        ];
    }

    protected function getPlotOptions(){
        $widget = $this->widget;
        $barOptions = [
            "horizontal" => true,
            "dataLabels" => [
                "position" => !empty($widget['chartInvertedAxes']) ? 'left' : 'top',
                "textAnchor" => "end",
            ],
        ];

        if(!empty($widget['chartXAxisBandDimensionGap'])){
            $barOptions["barHeight"] = $this->getBarHeightByDimensionGap($widget['chartXAxisBandDimensionGap']);
        }

        return [
            "bar" => $barOptions
        ];
    }

    protected function getBarWidth(){
        $widget = $this->widget;
        $barWidth = "auto";

        if(!empty($widget['chartXAxisBandDimensionGap'])){
            $barWidth = $this->getBarHeightByDimensionGap($widget['chartXAxisBandDimensionGap']);
        }

        return $barWidth;
    }
    
    protected function getBarHeightByDimensionGap($dimensionGap){
        return strval((1-$dimensionGap)*100 )."%";
    }

    protected function getSerieStacked(){
        $widget = $this->widget;

        if(!empty($widget['serieStacked'])){
            return $widget['serieStacked'];
        }
        return false;
    }

    protected function getChart(){
        $widget = $this->widget;

        return [
            "type" => "bar",
            "id" => $widget['id'],
            "stacked" => $this->getSerieStacked(),
            "toolbar" => [
                "show" => true,
                "tools" => [
                    "download" => true,
                    "selection" => false,
                    "zoom" => false,
                    "zoomin" => false,
                    "zoomout" => false,
                    "pan" => false,
                    "reset" => false,
                    "customIcons" => []
                ],
            ],
        ];
    }

    protected function getSerieColors(){
        $widget = $this->widget;
        if(!empty($widget["chartSeries"][0]["serieColors"])){
            return $widget["chartSeries"][0]["serieColors"];
        }
        return self::DEFAULT_COLORS;
    }

    protected function getDataLabels(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["serieShowValueLabels"]) ? $widget["serieShowValueLabels"] : false,
            "position" => "top",
            "textAnchor" => "start",
            "style" => [
                "colors" => $this->getSerieStacked() ? [] : $this->getSerieColors()
            ],
        ];
    }

    protected function getTitle(){
        $widget = $this->widget;

        return [
            "text" => $widget['name'],
            "textStyle" => [
                "fontSize" => "16px"
            ]
        ];
    }

    protected function getNoData(){
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
              "fontSize" => '14px',
            ],
        ];
    }

    protected function getLegend()
    {
        $widget = $this->widget;
        $string = $widget["chartLegendPosition"];
        $variable = [];
        $variations = explode(" ", $string);
        foreach ($variations as $variation) {
            if ($variation == "top") {
                $variable['top'] = "top";
                $variable['orient'] = "vertical";
            }
            if ($variation == "bottom") {
                $variable['bottom'] = "0%";
                $variable['orient'] = "vertical";
            }
            if ($variation == "left") {
                $variable['left'] = "left";
                $variable['orient'] = "vertical";
            }
            if ($variation == "right") {
                $variable['right'] = "right";
                $variable['orient'] = "vertical";
            }
        }
        return $variable;
    }

    protected function getOptions(){
        $widget = $this->widget;

        return [
            "grid" => [
                "left" => "30%",
                "bottom" => "10%"
            ],
            "color" => $this->getSerieColors(),
            // "drillDownView" => empty($widget['drillDownView']) ? null : $widget['drillDownView'],
            // Series: "barWidth" => $this->getBarWidth(),
            // Series: "dataLabels" => $this->getDataLabels(),
            // Series: "chart" => $this->getChart(),
            "legend" => $this->getLegend(),
            "title" => $this->getTitle(),
            "tooltip" => [
                "trigger" => "axis",
                "axisPointer" => [
                    "type" => "shadow"
                ]
            ],
            "xAxis" => $this->getXAxis(),
            "yAxis" => $this->getYAxis(),
            'series' => $this->getSeries(),
        ];
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_BAR,
            "component" => "ZdECharts",
            "chartType" => $widget['serieType'],
            "height" =>"100%",
            "invertedAxis" => true,
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions(),
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
