<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class PiePivot extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_PIE_PIVOT = "58762a38ca6bc3c1f61960c0";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getXAxis(){
        $widget = $this->widget;
        $data = $this->widgetData;
        $categoria = [];
        foreach ($data as $row) {
            $categoria[] = $row[$widget['chartXAxisField']];
        }
        $xAxis = [
            "mask" => !empty($widget['chartXAxisFormat']) ? $widget['chartXAxisFormat'] : null,
            "categories" => $categoria,
            "field" => $widget['chartXAxisField'],
            "labels" => [
                "show" => true,
                "rotate" => !empty($widget['chartXAxisTicksRotation']) ? $widget['chartXAxisTicksRotation']-90 : -60,
            ],
            "tickPlacement" => "between"
        ];

        if($xAxis["labels"]["rotate"] != 0){
            $xAxis["labels"]["rotateAlways"] = false;
        }

        return $xAxis;
    }

    protected function getYAxis(){
        $widget = $this->widget;
        $serieYAxisFormat = !empty($widget['chartSeries'][0]['serieValueFormat']) ? $widget['chartSeries'][0]['serieValueFormat'] : null;
        $yAxis = [
            "mask" => $serieYAxisFormat,
            "dataLabels" => [],
            "labels" => [
                "show" => true,
            ],
            "opposite" => !empty($widget["serieYAxisPosition"]) ? $widget["serieYAxisPosition"] == "right" : false,
        ];

        if(!empty($widget['serieYAxisLabel'])){
            $yAxis['title'] = [
                "text" => $widget['serieYAxisLabel'],
                "style" => [
                    "fontSize" => "16px"
                ]
            ];
        }

        return $yAxis;
    }

    protected function getSeries(){
        $widget = $this->widget;
        $series = [];
        foreach($widget["seriePivotedFields"] as $fact){
            $series[] = $this->widgetData[0][$fact];
        }
        return $series;
    }

    protected function getTooltip(){
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["chartShowTooltip"]) ? $widget["chartShowTooltip"] : false
        ];
    }

    protected function getPlotOptions(){
        $widget = $this->widget;
        $pieOptions = [
            "pie" => [
                "labels" => [
                    "show" => true,
                    "name" => [
                        "show" => true,
                    ],
                    "value" => [
                        "show" => true,
                    ],
                ]
            ]
        ];

        return [
            "pie" => $pieOptions
        ];
    }

    protected function getSerieStacked(){
        $widget = $this->widget;

        if(!empty($widget['serieStacked'])){
            return $widget['serieStacked'];
        }
        return false;
    }

    protected function getChart(){
        $widget = $this->widget;

        return [
            "type" => "bar",
            "id" => $widget['id'],
            "stacked" => $this->getSerieStacked(),
            "toolbar" => [
                "show" => true,
                "tools" => [
                    "download" => true,
                    "selection" => false,
                    "zoom" => false,
                    "zoomin" => false,
                    "zoomout" => false,
                    "pan" => false,
                    "reset" => false,
                    "customIcons" => []
                ],
            ],
        ];
    }

    protected function getSerieColors(){
        $widget = $this->widget;
        if(!empty($widget["chartSeries"][0]["serieColors"])){
            return $widget["chartSeries"][0]["serieColors"];
        }
        return self::DEFAULT_COLORS;
    }

    protected function getDataLabels(){
        $widget = $this->widget;
        $showDataLabel = false;
        $showAsPercent = false;
        if(!empty($widget["serieShowValueLabels"])){
            $showDataLabel = $widget["serieShowValueLabels"] != "false";
            $showAsPercent = $widget["serieShowValueLabels"] == "%";
        }
        return [
            "enabled" => $showDataLabel,
            "position" => "top",
            "textAnchor" => "middle",
            "showAsPercent" => $showAsPercent,
        ];
    }

    protected function getTitle(){
        $widget = $this->widget;

        return [
            "text" => $widget['name'],
            "style" => [
                "fontSize" => "16px"
            ]
        ];
    }

    protected function getNoData(){
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
              "fontSize" => '14px',
            ],
        ];
    }

    protected function getOptions(){
        $widget = $this->widget;

        $legendPosition = "top";
        if($widget["chartLegendPosition"]){
            $legendPosition = explode(" ",$widget["chartLegendPosition"])[0];
        }

        return [
            "colors" => $this->getSerieColors(),
            "drillDownView" => empty($widget['drillDownView']) ? null : $widget['drillDownView'],
            "plotOptions" => $this->getPlotOptions(),
            "dataLabels" => $this->getDataLabels(),
            "chart" => $this->getChart(),
            "labels" => $widget["seriePivotedLabels"],
            "legend" => [
                "show" => isset($widget["chartLegendClosed"]) ? $widget["chartLegendClosed"] !== "removeLegend" : true,
                "position" => $legendPosition
            ],
            "zoom" > [
                "enabled" => true
            ],
            "title" => $this->getTitle(),
            "noData" => $this->getNoData(),
            "tooltip" => $this->getTooltip(),
            "xaxis" => $this->getXAxis(),
            "yaxis" => $this->getYAxis()
        ];
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;

        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_PIE_PIVOT,
            "component" => self::CHART_COMPONENT,
            "chartType" => 'pie',
            "formatValue" => true,
            "height" =>"100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions(),
            "series" => $this->getSeries(),
            
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
